package camomile

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module CamomileLib.Database

Database.ml : Unified interfaces of stored data for Camomile

val escape : string -> string

read dir suffix reader key reads information using reader. Data are supposed to reside in the files under dir directory with suffix suffix. reader takes in_channel as an argument and read data from in_channel. The key specifies key associated the value. Any characters can be used in key, since they are properly escaped. If there are no data associated to key, raise Not_found.

module type FileReader = sig ... end
module Make (_ : FileReader) : sig ... end
val read : string -> string -> ('a -> 'b) -> string -> 'b
val write : string -> string -> (out_channel -> 'a -> unit) -> string -> 'a -> unit

writer dir suffix writer key data write data associated the key into the directory dir with suffix. You can use any characters in key since they are propery escaped.