package camomile

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Class type OOChannel.obj_output_channelSource

Generic output channel Have the same interface of Polymorphic output channel of http://www.ocaml-programming.de/rec/IO-Classes.html All channels of Camomile having this interface must confirm the behaviour defined in the recommendation above.

method close_out : unit -> unit

If close_oout cannot output all buffered objects, flush raises Failure

If flush cannot output all buffered objects, flush raises Failure

method flush : unit -> unit

If flush cannot output all buffered objects, flush raises Failure

method put : 'a -> unit